function [P,Q,E,b,obj] = DMLSR(X,L,Train_Lab,opts)

addpath(genpath('sub_func'))
miu = opts.miu;
rho = opts.rho;
max_mu = 1e8;
lambda1 = opts.lambda1;
lambda2 = opts.lambda2;
knn = opts.knn;
nnClass   = opts.nnClass;
max_iter  = opts.maxIter;

% Initialize
[dim,N] = size(X);
options = [];
options.ReducedDim = nnClass;
[P1,~] = PCA1(X',options);              
[Q,~] = LSR(X,L',lambda1);
E  = zeros(dim,N);
C1 = zeros(dim,N);
num_class = numel(unique(Train_Lab));
T = zeros(N,num_class);
for i = 1 : N
    T(i,Train_Lab(i))  = 1.0;
end
[L,W]=Construct_L(X',Train_Lab,knn); 
Ls = L;
temp = X*(Ls)*X';

for iter = 1:max_iter
    % ---------- Update P ----------- %
    if (iter == 1)
        P = P1;
    else
        M  = X+C1/miu;
        [U1,S1,V1] = svd(M*X'*Q,'econ');
        P = U1*V1';
        clear M;
    end
    % ---------- Update Q ----------- %
    M  = X+C1/miu;
    Q1 = lambda2*(temp'+temp)+2*lambda1*eye(dim)+(1+miu)*(X*X');
    Q2 = miu*X*M'*P+X*T;
    Q  = Q1\Q2;
    b = T - X'*Q;
    b = mean(b);
    % ---------- Update C1,mu ----------- %
    C1  = C1+miu*(X-P*Q'*X);
    miu  = min(rho*miu,max_mu);
    leq = X-P*Q'*X;
    % ---------- Update T ----------- %
    U = X'*Q + (b'*ones(1,N))';
    U_temp = U';
    for ind = 1:N
        T_temp(:,ind) = optimizedT(U_temp(:,ind), Train_Lab(ind));
    end
    T = T_temp';
    clear U;
    obj(iter) = norm(T-X'*Q,'fro').^2+lambda1*norm(Q,'fro').^2+lambda2.*trace(Q'*temp*Q);
    if iter >2
        if norm(leq, Inf) < 1e-7 && abs(obj(iter)-obj(iter-1))<1e-7
            iter
            break;
        end 
    end
end

end

function T = optimizedT(R, label)
classNum = length(R);
T = zeros(classNum,1);
V = R + 0.05 - repmat(R(label),classNum,1);  
step = 0;
num = 0;
for i = 1:classNum
    if i~=label
        dg = V(i);
        for j = 1:classNum
            if j~=label
                if V(i) < V(j)
                    dg = dg + V(i) - V(j);
                end
            end
        end
        if dg > 0
            step = step + V(i);
            num = num + 1;
        end
    end
end
step = step / (1+num);
for i = 1:classNum
    if i == label
        T(i) = R(i) + step;
    else
        T(i) = R(i) + min(step - V(i), 0);
    end
end
end
function [W, b] = LSR(X, Y, gamma)
[dim, N] = size (X);
[~, N] = size(Y);
XMean = mean(X')';
XX = X - repmat(XMean, 1, N);

W = [];
b = [];
t0 =  XX * XX' + gamma * eye(dim);
W = t0 \ (XX * Y');   
b = Y - W' * X;
b = mean(b')'; 
end
    

    